# -*- coding: utf-8 -*-

import sys
import os.path
import pygtk
if sys.platform != 'win32':
    pygtk.require('2.0')
import gtk


#
#
class FileSelectDialog:
    '''
    ファイル選択ダイアログクラス
    '''
    def __init__(self,
                 title='',                 
                 parent=None,
                 action=gtk.FILE_CHOOSER_ACTION_OPEN,
                 position=gtk.WIN_POS_CENTER):
        '''
        初期化
        '''
        buttons = (gtk.STOCK_CANCEL, gtk.RESPONSE_CANCEL,
                   gtk.STOCK_OK, gtk.RESPONSE_OK)
        
        self.chooser = gtk.FileChooserDialog(title,
                                             parent,
                                             action,
                                             buttons)

        self.chooser.set_position(position)

    def run(self):
        '''
        ファイル選択ダイアログのイベントループを開始する。
        この関数は、OK/Cancelボタンでダイアログが閉じられる
        まで、戻ってこない。

        戻り値:
           gtk.RESPONSE_OK    : OKボタンで閉じられた
           gtk.RESPONSE_CANCEL: Cancelボタンで閉じられた
        '''
        return self.chooser.run()

    def destroy(self):
        '''
        ファイル選択ダイアログを破棄する。
        '''
        self.chooser.destroy()

    def get_filenames(self):
        '''
        ファイル選択ダイアログで指定されたファイル名を取得する。

        戻り値:
           選択されたファイルのパスをリストとして返す。
        '''
        names = []
        if self.chooser.get_select_multiple():
            names = self.chooser.get_filenames()
        else:
            n = self.chooser.get_filename()
            names = [n]
        return names

    def set_filter_info(self, infoList):
        '''
        ファイル選択ダイアログにファイルフィルタを設定する。
        
        infoList
            ファイルフィルタを設定するための情報。
            タプル(フィルタ名、フィルタパターン1, ... ,フィルタパターンN)
            を要素にもつシーケンス（タプルまたはリスト）で指定する必要が
            ある。
        '''
        for info in infoList:
            if len(info) < 2:
                continue

            # フィルタを生成
            ff = gtk.FileFilter()
            
            # infoの最初の要素をフィルタ名称として設定
            ff.set_name(info[0])
            
            # infoの２番目以降をフィルタパターンとして設定
            for pat in info[1:]:
                ff.add_pattern(pat)
                
            # chooserにフィルタを追加
            self.chooser.add_filter(ff)

    def set_action(self, action):
        '''
        ファイル選択ダイアログにアクションを設定する。
        
        action:
            gtk.FileChooserDialog()で指定できるactionと同じ。
        '''
        self.chooser.set_action(action)

    def set_current_folder(self, folderPath):
        '''
        ファイル選択ダイアログに現在のフォルダ位置を設定する。
        
        action:
            gtk.FileChooserDialog()で指定できるactionと同じ。
        '''
        if folderPath == None:
            return
        if os.path.isdir(folderPath) == False:
            return
        folderPath = os.path.normpath(os.path.expanduser(folderPath))
        folderPath = os.path.abspath(folderPath)
        self.chooser.set_current_folder(folderPath)

    def get_current_folder(self):
        '''
        ファイル選択ダイアログで選択されていたフォルダパスを
        返す。
        '''
        return self.chooser.get_current_folder()

    def set_select_multiple(self, flag):
        '''
        ファイル選択ダイアログに複数ファイル選択可能
        かどうかを設定する。
        
        flag:
            True = 複数選択可能
        '''
        self.chooser.set_select_multiple(flag)

    def set_overwrite_confirmation(self, flag):
        '''
        ファイル選択ダイアログに上書き確認を表示する
        かどうかを設定する。
        
        flag:
            True = 上書き確認ダイアログを表示する
        '''
        self.chooser.set_do_overwrite_confirmation(flag)

#
#
def test_default_dialog():
    dlg = FileSelectDialog(title='Open dialog(default args)')
    response = dlg.run()

    if response == gtk.RESPONSE_OK:
        print '---- Open Dialog(default args) ----'
        print 'OK'
        print dlg.get_filenames()
        print dlg.get_current_folder()
    else:
        print 'Cancel'

#
#
def test_open_single_dialog():
    dlg = FileSelectDialog(title='Open single dialog',
                           action=gtk.FILE_CHOOSER_ACTION_OPEN)
    response = dlg.run()

    if response == gtk.RESPONSE_OK:
        print '---- Open single dialog ----'
        print 'OK'
        print dlg.get_filenames()
        print dlg.get_current_folder()
    else:
        print 'Cancel'

#
#
def test_open_multiple_dialog():
    dlg = FileSelectDialog(title='Open multiple dialog',
                           action=gtk.FILE_CHOOSER_ACTION_OPEN)
    dlg.set_select_multiple(True)
    response = dlg.run()

    if response == gtk.RESPONSE_OK:
        print '---- Open multiple dialog ----'
        print 'OK'
        print dlg.get_filenames()
        print dlg.get_current_folder()
    else:
        print 'Cancel'

#
#
def test_open_multiple_filtered_dialog():
    dlg = FileSelectDialog(title='Open multiple dialog(with filter)',
                           action=gtk.FILE_CHOOSER_ACTION_OPEN)

    filterInfo = [('image', '*.jpg', '*.tiff', '*.png'),
                  ('python', '*.py', '*.pyc'),
                  ('backup', '*.bak', '*.org', '*.*~')]
    dlg.set_filter_info(filterInfo)
    dlg.set_select_multiple(True)
    response = dlg.run()

    if response == gtk.RESPONSE_OK:
        print '---- Open multiple dialog(with filter) ----'
        print 'OK'
        print dlg.get_filenames()
        print dlg.get_current_folder()
    else:
        print 'Cancel'

#
#
def test_save_dialog():
    dlg = FileSelectDialog(title='Save dialog',
                           action=gtk.FILE_CHOOSER_ACTION_SAVE)
    response = dlg.run()

    if response == gtk.RESPONSE_OK:
        print '---- Save dialog ----'
        print 'OK'
        print dlg.get_filenames()
        print dlg.get_current_folder()
    else:
        print 'Cancel'

#
#
def test_save_overwrite_dialog():
    dlg = FileSelectDialog(title='Save dialog(overwrite confirmation)',
                           action=gtk.FILE_CHOOSER_ACTION_SAVE)
    dlg.set_overwrite_confirmation(True)
    response = dlg.run()

    if response == gtk.RESPONSE_OK:
        print '---- Save dialog(overwrite confirmation) ----'
        print 'OK'
        print dlg.get_filenames()
        print dlg.get_current_folder()
    else:
        print 'Cancel'

#
#
if __name__ == '__main__':
    #test_default_dialog()
    #test_open_single_dialog()
    #test_open_multiple_dialog()
    #test_open_multiple_filtered_dialog()
    #test_save_dialog()
    test_save_overwrite_dialog()
