# -*- coding: utf-8 -*-

from xml.dom.minidom import Document
from xml.sax.handler import ContentHandler


# タグ用　文字列定数
ELEMENT_NAME_ROOT = 'fcp_doc'
ELEMENT_NAME_TARGET = 'target'
ELEMENT_NAME_TARGET_SRC = 'src'
ELEMENT_NAME_TARGET_DST = 'dst'
ELEMENT_ATTR_PATH = 'path'


#
#
class FCPDocGenerator:
    '''
    pyFCP用のXMLテキストを生成するためのクラス。
    '''
    def __init__(self):
        '''
        初期化
        '''
        # MINI DOMのドキュメントを生成し、
        # 最上位のエレメントを１つ登録しておく。
        self.doc = Document()
        self.root = self.doc.createElement(ELEMENT_NAME_ROOT)
        self.doc.appendChild(self.root)

    def __del__(self):
        '''
        破棄
        '''
        self.doc.unlink()
        
    def generate(self, targetList, encode='utf-8'):
        '''
        xmlテキストの生成。
        targetListは2要素のタプル(srcPath, dstPath)のリスト
        を期待する。
        生成されるテキストのエンコードはencodeも指定可能。
        '''
        if self.doc == None or self.root == None:
            return

        for src, dst in targetList:
            elm = self.doc.createElement(ELEMENT_NAME_TARGET)
            self.root.appendChild(elm)
            
            # コピー元ファイルパス用のタグを追加
            if src:
                child = self.doc.createElement(ELEMENT_NAME_TARGET_SRC)
                child.setAttribute(ELEMENT_ATTR_PATH, src)
                elm.appendChild(child)
                
            # コピー先ファイルパス用のタグを追加
            if dst:
                child = self.doc.createElement(ELEMENT_NAME_TARGET_DST)
                child.setAttribute(ELEMENT_ATTR_PATH, dst)
                elm.appendChild(child)

        # xmlテキストを作成
        return self.doc.toprettyxml(encoding=encode)

#
#
class FCPDocHandler(ContentHandler):
    '''
    pyFCP用XMLをパースするためのクラス
    '''
    def __init__(self):
        '''
        生成
        '''
        ContentHandler.__init__(self)

        # 処理結果を保持するためのリスト
        self.targets = []
        
        # <target>タグが登場するたびに生成される２要素のリスト
        # として使用される。
        self.tmpTarget = None

        # 開始タグごとに実行する関数の辞書
        self.startFuncDict = {ELEMENT_NAME_TARGET: self.target_tag_start_func,
                              ELEMENT_NAME_TARGET_SRC: self.target_src_tag_start_func,
                              ELEMENT_NAME_TARGET_DST: self.sarget_dst_tag_start_func}
        
        # 終了タグごとに実行する関数の辞書
        self.endFuncDict = {ELEMENT_NAME_TARGET: self.target_tag_end_func}

    def get_targets(self):
        '''
        処理結果をを取得する。
        処理結果は[(srcPath, dstPath), ..., (srcPath, dstPath)]
        の形式のリスト。
        '''
        return self.targets
    
    def startElement(self, name, attrs):
        '''
        開始タグが出現するたびにパーサから呼び出される
        '''
        if name not in self.startFuncDict:
            return

        func = self.startFuncDict[name]
        func(attrs)

    def endElement(self, name):
        '''
        終了タグが出現するたびにパーサから呼び出される
        '''
        if name not in self.endFuncDict:
            return
        func = self.endFuncDict[name]
        func()

    def target_tag_start_func(self, attrs):
        '''
        <target>開始タグが出現した場合の処理
        '''
        # テンポラリのリストを生成。
        # リストは[srcPath, dstpath]の2要素を格納するために
        # に使用する。
        self.tmpTarget = ['', '']

    def target_src_tag_start_func(self, attrs):
        '''
        <src>タグが出現した場合の処理
        '''
        if self.tmpTarget == None:
            return
        self.tmpTarget[0] = attrs.getValue(ELEMENT_ATTR_PATH)
    
    def sarget_dst_tag_start_func(self, attrs):
        '''
        <dst>タグが出現した場合の処理
        '''
        if self.tmpTarget == None:
            return
        self.tmpTarget[1] = attrs.getValue(ELEMENT_ATTR_PATH)
    
    def target_tag_end_func(self):
        '''
        <target>終了タグが出現した場合の処理
        '''
        # <target></target>内に存在した
        # [srcPath, dstPath]のペアをタプルに変換し
        # 処理結果用のリストに格納する。
        if self.tmpTarget == None:
            return
        self.targets.append(tuple(self.tmpTarget))
        self.tmpTarget = None
